import time
import uuid
import exceptions
import json
import traceback
import yaml
import socket
import importlib
import datetime

from twisted.application import service
from twisted.internet import task, reactor, protocol, defer
from twisted.python import log

from txredis.client import RedisClient, RedisSubscriber


class RhumbaQueue(object):
    def __init__(self, config, svc):
        self.config = config
        self.service = svc

        self.name = config['name']
        self.expire = int(config.get('expire', 3600))
        self.plugin = self.loadPlugin(config['plugin'])

        self.inter = float(config.get('inter', 1))

        self.fast_inter = float(config.get('fast_inter', 0.1))

        self.max_jobs = config.get('max_jobs', 5)
        self.jobs = 0
        self.cycl = 0

        self.t = task.LoopingCall(self.tick)

    def loadPlugin(self, plugin):
        try:
            return getattr(importlib.import_module(plugin), 'Plugin')(self.config)
        except exceptions.ImportError, e:
            log.msg("Error importing plugin %s : %s" % (plugin, repr(e)))
            return None
        
    @defer.inlineCallbacks
    def grabQueue(self):
        item = yield self.service.client.rpop("rhumba.q.%s" % self.name)
        if item:
            defer.returnValue(json.loads(item))
        else:
            defer.returnValue(None)

    @defer.inlineCallbacks
    def processQueue(self, item):
        m = item.get('message')
        if m:
            uid = item['id']
            yield self.service.setStatus("processing:%s:%s:%s" % (
                m, uid, time.time()))

            result = yield self.processItem(m, item.get('params', {}))

            d = {
                'result': result,
                'time': time.time()
            }

            yield self.service.client.set('rhumba.q.%s.%s' % (self.name, uid),
                json.dumps(d), expire=self.expire)

    @defer.inlineCallbacks
    def reQueue(self, request):
        response = yield self.service.client.lpush("rhumba.q.%s" % self.name)

    def processItem(self, message, params):
        fn = getattr(self.plugin, 'call_%s' % message)

        return defer.maybeDeferred(fn, params)

    @defer.inlineCallbacks
    def queueRun(self):
        if self.cycl > 0:
            self.cycl -= 1
            defer.returnValue(None)

        if self.jobs >= self.max_jobs:
            defer.returnValue(None)
        
        item = yield self.grabQueue()

        if item:
            self.jobs += 1
            yield self.processQueue(item)
            self.jobs -= 1
        else:
            self.cycl = self.inter/self.fast_inter

            yield self.service.setStatus("ready")

        defer.returnValue(None)

    def tick(self):
        reactor.callLater(self.fast_inter, self.queueRun)

    def startQueue(self):
        """Starts the timer for this queue"""
        self.td = self.t.start(self.fast_inter)

    def stopQueue(self):
        """Stops the timer for this queue"""
        self.td = None
        self.t.stop()


class RhumbaService(service.Service):
    """ Rhumba service
    Runs timers, configures sources and and manages the queue
    """
    def __init__(self, config):
        try:
            self.config = yaml.load(config)
        except:
            self.config = {}

        self.uuid = uuid.uuid1().get_hex()

        self.hostname = socket.gethostbyaddr(socket.gethostname())[0]

        self.redis_host = self.config.get('redis_host', 'localhost')
        self.redis_port = int(self.config.get('redis_port', 6379))

        self.expire = 3600
        
        self.queues = {}

        self.crons = {}

        self.t = task.LoopingCall(self.heartbeat)

    @defer.inlineCallbacks
    def checkCrons(self, now):

        for queue, crons in self.crons.items():
            runner = yield self.checkCronRunners(queue)

            if not runner:
                yield self.registerCronRunner(queue)
                runner = self.uuid

            if runner == self.uuid:
                for cron in crons:
                    lastRun = yield self.lastRun(queue, cron.name)

                    if lastRun:
                        lastRun = int(lastRun)

                    if cron.checkCron(lastRun, now):
                        plug = self.queues[queue].plugin
                        yield self.setLastRun(queue, cron.name, now)
                        d = {
                            'id': uuid.uuid1().get_hex(),
                            'version': 1,
                            'message': cron.name.split('call_', 1)[-1],
                            'params': {}
                        }

                        # Queue this job
                        log.msg('Queing %s scheduled job %s' % (queue, repr(d)))
                        yield self.client.lpush(
                            'rhumba.q.%s' % queue, json.dumps(d))
   
    @defer.inlineCallbacks
    def heartbeat(self):

        yield self.checkCrons(datetime.datetime.now())

        yield self.client.set(
            "rhumba.server.%s.heartbeat" % self.hostname, time.time(), expire=self.expire)

    def setStatus(self, status):
        return self.client.set(
            "rhumba.server.%s.status" % self.hostname, status, expire=self.expire)

    def startBeat(self):
        self.td = self.t.start(1.0)

    def lastRun(self, queue, fn):
        return self.client.get("rhumba.crons.%s.%s" % (queue, fn))

    def setLastRun(self, queue, fn, now):
        now = time.mktime(now.timetuple())
        return self.client.set("rhumba.crons.%s.%s" % (queue, fn), now)

    def registerCronRunner(self, queue):
        return self.client.set("rhumba.crons.%s" % queue, self.uuid, expire=60)

    def deregisterCronRunner(self, queue):
        return self.client.delete("rhumba.crons.%s" % queue)

    def checkCronRunners(self, queue):
        return self.client.get("rhumba.crons.%s" % queue)

    def setupQueues(self):
        queues = self.config.get('queues', [])
        for queue in queues:
            q = RhumbaQueue(queue, self)
            qname = queue['name']
            self.queues[qname] = q

            if q.plugin:
                # Find all methods decorated with @cron
                crons = [
                    getattr(q.plugin, i).cronable for i in dir(q.plugin)
                    if hasattr(getattr(q.plugin, i), 'cronable')
                ]

                if crons:
                    self.crons[qname] = crons

    @defer.inlineCallbacks
    def startService(self):
        clientCreator = protocol.ClientCreator(reactor, RedisClient)
        self.client = yield clientCreator.connectTCP(
            self.redis_host, self.redis_port)
        
        log.msg('Starting Rhumba')
    
        reactor.callWhenRunning(self.startBeat)
        self.setupQueues()
        queues = 0
        for k, v in self.queues.items():
            if v.plugin:
                queues += 1
                log.msg('Starting queue %s: plugin=%s' % (k, v.plugin))
                reactor.callWhenRunning(v.startQueue)

        if queues < 1:
            log.msg('No queues are running')

def makeService(config):
    return RhumbaService(config)




import datetime
import calendar

Sunday = 0
Monday = 1
Tuesday = 2
Wednesday = 3
Thursday = 4
Friday = 5
Saturday = 6

class Cronable(object):
    def __init__(self, name, secs, min, hour, day, month, weekday):
        self.name = name
        self.secs = secs
        self.min = min
        self.hour = hour
        self.day = day
        self.month = month
        self.weekday = weekday

    def _checkEvery(self, t, n, total, wrap=59, exact=True, repeat=False):
        if not t:
            return False, False
        tseg = t

        if isinstance(t, str):
            if t[:2] == '*/':
                tseg = int(t[2:])
                if total >= tseg:
                    return True, True
                return True, False
            else:
                tseg = int(t)
       
        if exact:
            if (tseg == n) and ((total >= wrap) or repeat):
                return False, True
        else:
            if (n >= tseg) and ((total >= wrap) or repeat):
                return False, True

        return False, False

    def _checkSecs(self, now, delta, repeat=False):
        total_secs = delta.total_seconds()

        return self._checkEvery(
            self.secs, now.second, total_secs, exact=False, repeat=repeat)

    def _checkMins(self, now, delta, repeat=False):
        total_mins = delta.total_seconds() / 60

        return self._checkEvery(
            self.min, now.minute, total_mins, exact=False, repeat=repeat)

    def _checkHours(self, now, delta, repeat=False):
        total_hours = delta.total_seconds() / (60*60)

        return self._checkEvery(
            self.hour, now.hour, total_hours, wrap=23, repeat=repeat)
    
    def _checkDays(self, now, delta, repeat=False):
        total_days = delta.total_seconds() / (60*60*24)

        # Get the max days in this month
        start, end = calendar.monthrange(now.year, now.month)

        return self._checkEvery(
            self.day, now.day, total_days, wrap=end, repeat=repeat)

    def _checkMonth(self, now, delta, repeat=False):
        # Get the max days in this month
        if now.month == 1:
            lastm = 12
            year = now.year - 1
        else:
            lastm = now.month - 1
            year = now.year

        start, end = calendar.monthrange(year, lastm)

        total_days = delta.total_seconds() / (60*60*24*end)

        return self._checkEvery(self.month, now.month, total_days, wrap=12,
            exact=True, repeat=repeat)

    def _checkWeekday(self, now, delta, repeat=False):
        if isinstance(self.weekday, str):
            self.weekday = int(self.weekday)

        if (self.weekday == now.isoweekday()) and ((delta.days > 7) or repeat):
            return False, True
        else:
            return False, False

    def checkCron(self, last, now):
        assert(isinstance(now, datetime.datetime))

        if not last:
            last = 0 

        last_date = datetime.datetime.fromtimestamp(float(last))
        delta = now - last_date

        run = False

        chk = [
            (self.secs, self._checkSecs),
            (self.min, self._checkMins),
            (self.hour, self._checkHours),
            (self.weekday, self._checkWeekday),
            (self.day, self._checkDays),
            (self.month, self._checkMonth),
        ]

        every = False

        for a, b in chk:
            if a:
                e, run = b(now, delta, repeat=every)
                every = (run and e) or every

        return run
            

def cron(secs=None, min=None, hour=None, day=None, month=None, weekday=None):
    def wrapper(fn):
        fn.cronable = Cronable(
            fn.__name__, secs, min, hour, day, month, weekday)
        return fn
    return wrapper



from zope.interface import implements
 
from twisted.python import usage
from twisted.plugin import IPlugin
from twisted.application.service import IServiceMaker
 
from rhumba import service
 
class Options(usage.Options):
    optParameters = [
        ["config", "c", "rhumba.yml", "Config file"],
    ]
 
class RhumbaServiceMaker(object):
    implements(IServiceMaker, IPlugin)
    tapname = "rhumba"
    description = "An asynchronous job queue"
    options = Options
 
    def makeService(self, options):
        try:
            config = open(options['config'])
        except:
            config = "{}"
        return service.makeService(config)
 
serviceMaker = RhumbaServiceMaker()



import redis
import time
import json
import uuid

from twisted.internet import reactor, defer

from txredis.client import RedisClient

class AsyncRhumbaClient(object):
    """
    Rhumba client
    """
    def __init__(self, host='localhost', port=6379, db=0):
        self.redis_host = host
        self.redis_port = port
        self.db = db

        reactor.callWhenRunning(self.connect)

    @defer.inlineCallbacks
    def connect(self):
        clientCreator = protocol.ClientCreator(reactor, RedisClient, db=self.db)

        self.client = yield clientCreator.connectTCP(
            self.redis_host, self.redis_port
        )

    @defer.inlineCallbacks
    def queue(self, queue, message, params={}):
        """
        Queue a job in Rhumba
        """
        d = {
            'id': uuid.uuid1().get_hex(),
            'version': 1,
            'message': message,
            'params': params
        }

        yield self.client.lpush('rhumba.q.%s' % queue, json.dumps(d))

        defer.returnValue(d['id'])
    
    @defer.inlineCallbacks
    def getResult(self, queue, uid):
        """
        Retrieve the result of a job from its ID
        """
        r = yield self.client.get('rhumba.q.%s.%s' % (queue, uid))

        if r:
            defer.returnValue(json.loads(r))
        else:
            defer.returnValue(None)

    @defer.inlineCallbacks
    def clusterStatus(self):
        """
        Returns a dict of cluster nodes and their status information
        """
        servers = self.client.keys('rhumba.server.*.heartbeat')
        
        d = {}

        now = time.time()

        for s in servers:
            sname = s.split('.', 2)[-1].rsplit('.', 1)[0]
            
            last = yield self.client.get('rhumba.server.%s.heartbeat' % sname)

            if not last:
                last = 0

            status = yield self.client.get('rhumba.server.%s.status' % sname)

            if (status == 'ready') and (now - last > 5):
                status = 'offline'

            d[sname] = {
                'lastseen': last,
                'status': status
            }

        defer.returnValue(d)

class RhumbaClient(object):
    """
    Rhumba client
    """
    def __init__(self, host='localhost', port=6379, db=0):
        self.host = host
        self.port = port
        self.db = db

    def _get_client(self):
        return redis.StrictRedis(host=self.host, port=self.port, db=self.db)

    def queue(self, queue, message, params={}):
        """
        Queue a job in Rhumba
        """
        d = {
            'id': uuid.uuid1().get_hex(),
            'version': 1,
            'message': message,
            'params': params
        }

        self._get_client().lpush('rhumba.q.%s' % queue, json.dumps(d))
        return d['id']

    def getResult(self, queue, uid):
        """
        Retrieve the result of a job from its ID
        """
        r = self._get_client().get('rhumba.q.%s.%s' % (queue, uid))

        if r:
            return json.loads(r)
        else:
            return None

    def clusterStatus(self):
        """
        Returns a dict of cluster nodes and their status information
        """
        c = self._get_client()
        servers = c.keys('rhumba.server.*.heartbeat')
        
        d = {}

        now = time.time()

        for s in servers:
            sname = s.split('.', 2)[-1].rsplit('.', 1)[0]

            last = float(c.get('rhumba.server.%s.heartbeat' % sname))
            status = c.get('rhumba.server.%s.status' % sname)

            if (status == 'ready') and (now - last > 5):
                status = 'offline'

            d[sname] = {
                'lastseen': last,
                'status': status
            }

        return d



import yaml
import json
import datetime

from twisted.trial import unittest

from twisted.internet import defer, reactor, error
from twisted.internet.endpoints import TCP4ClientEndpoint, connectProtocol

from rhumba import service, client

test_config = {
    'redis_host': 'localhost',
    'queues': [{
        'id': 0, 'max_jobs': 1,
        'name': 'testqueue',
        'plugin': 'rhumba.tests.plugin'
    }],
    'redis_port': 6379
}

class FakeRedis(object):
    """Fake redis client object
    """
    def __init__(self):
        self.kv = {}

    def lpush(self, key, val, expire=None):
        if key in self.kv:
            self.kv[key].insert(0, val)
        else:
            self.kv[key] = [val]

    def set(self, key, val, expire=None):
        self.kv[key] = val

    def get(self, key):
        return self.kv.get(key, None)

    def rpop(self, key):
        if (key in self.kv) and self.kv[key]:
            return self.kv[key].pop(-1)
        else:
            return None

class AsyncWrapper(object):
    def __init__(self, c):
        self.c = c

    def lpush(self, *a, **kw):
        return defer.maybeDeferred(self.c.lpush, *a)

    def rpop(self, *a, **kw):
        return defer.maybeDeferred(self.c.rpop, *a)

    def set(self, *a, **kw):
        return defer.maybeDeferred(self.c.set, *a)

    def get(self, *a, **kw):
        return defer.maybeDeferred(self.c.get, *a)


class RhumbaTest(unittest.TestCase):
    def setUp(self):
        self.service = service.RhumbaService(yaml.dump(test_config))
        self.client = FakeRedis()
        self.service.setupQueues()
        self.service.client = AsyncWrapper(self.client)

        self.c = client.RhumbaClient()
        self.c._get_client = lambda : self.client


class TestClient(client.AsyncRhumbaClient):
    def __init__(self, client, *a, **kw):
        self.client = client

    def connect(self):
        pass

class TestService(RhumbaTest):

    @defer.inlineCallbacks
    def test_async_client(self):
        client = TestClient(self.service.client)

        yield client.queue('testqueue', 'test')
        
        message = self.service.client.c.kv['rhumba.q.testqueue'][0]

        self.assertIn('test', message)

    @defer.inlineCallbacks
    def test_heartbeat(self):
        yield self.service.heartbeat()

        hb = self.client.kv.get("rhumba.server.%s.heartbeat" % self.service.hostname)

        self.assertTrue(hb is not None)
    
    @defer.inlineCallbacks
    def test_status(self):
        yield self.service.setStatus('test')

        hb = self.client.kv.get("rhumba.server.%s.status" % self.service.hostname)
        self.assertEquals(hb, 'test')

    def test_queues(self):
        self.assertTrue('testqueue' in self.service.queues)

    def _wait_for(self, queue, uid):
        d = defer.Deferred()
        def poll(d):
            result = self.c.getResult(queue, uid)

            if result:
                d.callback(result)
            else:
                reactor.callLater(0.1, poll, d)

        reactor.callLater(0.1, poll, d)
        return d

    @defer.inlineCallbacks
    def test_call(self):
        queue = self.service.queues['testqueue']

        uuid1 = self.c.queue('testqueue', 'test', {'count': 1, 'delay': 2})
        uuid2 = self.c.queue('testqueue', 'test', {'count': 2, 'delay': 1})

        reactor.callLater(0, queue.queueRun)
        reactor.callLater(1, queue.queueRun)

        result = yield self._wait_for('testqueue', uuid1)
        self.assertEquals(result['result'], None)

        result = self.c.getResult('testqueue', uuid2)
        self.assertEquals(result, None)

        reactor.callLater(0, queue.queueRun)

        result = yield self._wait_for('testqueue', uuid2)
        
        self.assertEquals(result['result'], None)


    def setUp(self):
        self.service = service.RhumbaService(yaml.dump(test_config))
        self.client = FakeRedis()
        self.service.setupQueues()
        self.service.client = AsyncWrapper(self.client)

        self.c = client.RhumbaClient()
        self.c._get_client = lambda : self.client


class TestCron(RhumbaTest):
    def _messages(self):
        """ Retrieve a list of only messages in the queue """
        q = [json.loads(i)['message'] for i in self.client.kv.get(
            'rhumba.q.testqueue', [])]
        return q

    def _flush(self):
        self.client.kv['rhumba.q.testqueue'] = []

    @defer.inlineCallbacks
    def test_cron(self):
        queue = self.service.queues['testqueue']

        yield self.service.checkCrons(datetime.datetime.now())
        self.assertIn('rhumba.crons.testqueue.call_everysecond', self.client.kv)
        self.assertIn('everysecond', self._messages())

    @defer.inlineCallbacks
    def test_cron_repeat(self):
        queue = self.service.queues['testqueue']

        yield self.service.checkCrons(datetime.datetime.now())
        self.assertIn('everysecond', self._messages())

    @defer.inlineCallbacks
    def test_cron_backwards_clock(self):
        queue = self.service.queues['testqueue']
        # Clock goes backwards
        yield self.service.checkCrons(datetime.datetime(2015, 3, 3, 5, 3, 0))
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 2, 3, 5, 3, 2))
        self.assertNotIn('everytwentythsecond', self._messages())

        yield self.service.checkCrons(datetime.datetime(2015, 1, 3, 5, 3, 20))
        self.assertIn('everytwentythsecond', self._messages())


    @defer.inlineCallbacks
    def test_cron_seconds(self):
        queue = self.service.queues['testqueue']

        now = datetime.datetime(2015, 3, 3, 4, 1, 1)

        yield self.service.checkCrons(now)
        self.assertNotIn('everytwentythsecond', self._messages())
        self._flush()

        # 20 second mark
        yield self.service.checkCrons(datetime.datetime(2015, 3, 3, 4, 1, 20))
        self.assertIn('everytwentythsecond', self._messages())
        self._flush()
        
        yield self.service.checkCrons(datetime.datetime(2015, 3, 3, 4, 1, 21))
        self.assertNotIn('everytwentythsecond', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 3, 3, 4, 1, 32))
        self.assertNotIn('everytwentythsecond', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 4, 3, 20))
        self.assertIn('everytwentythsecond', self._messages())

    @defer.inlineCallbacks
    def test_cron_mins(self):
        queue = self.service.queues['testqueue']

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 4, 2, 1))
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 4, 5, 1))
        self.assertNotIn('everytenminutes', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 4, 12, 1))
        self.assertIn('everytenminutes', self._messages())

    @defer.inlineCallbacks
    def test_cron_hour(self):
        queue = self.service.queues['testqueue']

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 10, 2, 1))
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 11, 2, 2))
        self.assertIn('everyhour', self._messages())
        self.assertNotIn('atlunch', self._messages())

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 12, 12, 1))
        self.assertIn('everyhour', self._messages())
        self.assertIn('atlunch', self._messages())

    @defer.inlineCallbacks
    def test_cron_mins_hour(self):
        queue = self.service.queues['testqueue']

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 10, 2, 1))
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 14, 0, 0))
        self.assertIn('everytenminutesat2pm', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 14, 12, 0))
        self.assertIn('everytenminutesat2pm', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 14, 14, 0))
        self.assertNotIn('everytenminutesat2pm', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 15, 14, 0))
        self.assertNotIn('everytenminutesat2pm', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 15, 25, 0))
        self.assertNotIn('everytenminutesat2pm', self._messages())
        self._flush()

    @defer.inlineCallbacks
    def test_cron_multi(self):
        queue = self.service.queues['testqueue']

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 10, 2, 1))
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 10, 6, 10, 30, 0))
        self.assertIn('everyhour', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 10, 6, 12, 32, 1))
        self.assertIn('everytwohoursontuesday', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 10, 6, 12, 33, 1))
        self.assertNotIn('everytwohoursontuesday', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 10, 6, 14, 34, 1))
        self.assertIn('everytwohoursontuesday', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 10, 6, 14, 30, 20))
        self.assertNotIn('everytwohoursontuesday', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 10, 7, 14, 12, 1))
        self.assertNotIn('everytwohoursontuesday', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 10, 7, 14, 30, 1))
        self.assertNotIn('everytwohoursontuesday', self._messages())
        self._flush()

    @defer.inlineCallbacks
    def test_cron_month(self):
        queue = self.service.queues['testqueue']

        yield self.service.checkCrons(datetime.datetime(2015, 4, 3, 10, 2, 1))

        self.assertNotIn('december', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 12, 1, 0, 0, 0))
        self.assertIn('december', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 12, 2, 0, 0, 0))
        self.assertNotIn('december', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2015, 12, 31, 23, 59, 59))
        self.assertNotIn('december', self._messages())
        self._flush()

        yield self.service.checkCrons(datetime.datetime(2016, 12, 1, 0, 0, 0))
        self.assertIn('december', self._messages())



from setuptools import setup, find_packages


setup(
    name="rhumba",
    version='0.1.4',
    url='http://github.com/calston/rhumba',
    license='MIT',
    description="An asynchronous job queue written in Twisted",
    author='Colin Alston',
    author_email='colin.alston@gmail.com',
    packages=find_packages() + [
        "twisted.plugins",
    ],
    package_data={
        'twisted.plugins': ['twisted/plugins/rhumba_plugin.py']
    },
    include_package_data=True,
    install_requires=[
        'Twisted',
        'txredis',
        'PyYaml',
        'redis'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: System :: Distributed Computing',
    ],
)



"""
Test plugin for tests
"""
from twisted.internet import defer, reactor
from rhumba import RhumbaPlugin, cron, crontab

def sleep(secs):
    d = defer.Deferred()
    reactor.callLater(secs, d.callback, None)
    return d

class Plugin(RhumbaPlugin):

    @defer.inlineCallbacks
    def call_test(self, args):
        self.log("Test call %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(secs='*/1')
    def call_everysecond(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(secs=20)
    def call_everytwentythsecond(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(min='*/10')
    def call_everytenminutes(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(min='*/10', hour=14)
    def call_everytenminutesat2pm(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(hour='*/1')
    def call_everyhour(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(month=12)
    def call_december(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(hour=12)
    def call_atlunch(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(min='30', hour='*/2', weekday=crontab.Tuesday)
    def call_everytwohoursontuesday(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)

    @cron(min='30', hour='*/2', weekday=crontab.Tuesday, month='*/2')
    def call_every2hoursontuesdayevery2months(self, args):
        self.log("Test cron %s" % repr(args))
        yield sleep(args.get('delay', 1))
        defer.returnValue(None)
